#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "clique.h"

/* S.H.Whitesides algorithm for finding clique cut-sets.
** An Algorithm for Finding Clique Cut-Sets.
** Information Processing letters. Vol. 12, 1981. pp.31-32*/
/******************************************************************************
*** Function:    isCliqueSep
*** Description: Function checks if graph A contains a clique
***		 that separates the graph
*** Inputs:      G - the graph
***              Cl - space to save clique vertices
***              size- size of G 
*** Return Value: size of the clique
*** Side Effects: updates Cl to contain clique
*** Created:     Kerri Morgan   14 December, 2006
*** Modified:    Kerri Morgan  	25 May, 2007
***		 No longer returns size, if the entire graph is a clique
			Zoe Bukovac	12th May 2011
			Removed fixed size of graphs.
			Changed function name from isClique to isCliqueSep
*** Comments:    
*** Acknowledgements:
*******************************************************************************/
int isCliqueSep(int **A, int size, int *Cl)
{
	int v1, v2;
	int i,j,k;
	int N[size];
	int S[size];
	int M[size];
	int P[size];
	int W[size];
	int C[size];
	int R[size];
	int SSize=0;
	int visited[size];
	int x,y;
	int found =0;

	/* initialise adjacency matrix */
	for(i=0; i<size; i++)
	{
		S[i]=0;
		M[i]=0;
		N[i]=0;
		P[i]=0;
		W[i]=0;
		Cl[i]=-1;
		
	}
	
	/* step 0 */
	v1=0;
	v2=0;
	/* if vertex i has no neighbour j */
	for(i=0; i<size; i++)
	{
		for(j=i+1; j<size; j++)
		{
			if(A[i][j]==0)
			{
				v1=i;
				v2=j;
				i=size;
				break;
			}
		}
	}
	if(v1==v2) /* both 0 */
	{
		for(i=0; i<size; i++)
		{
			Cl[i]=i;
		}
		return 0; /* changed - I want a clique-separator */
		//return size;	/* entire graph is a clique */
	}

	/** TEST::
	printf("%d %d are v1 and v2\n", v1, v2);
**/	
	/* don't use neighbours of v1 for forming W*/
	for(i=0; i<size; i++)
	{
		visited[i]=A[v1][i];
	}

	/* find W */
	W[v2]=1;
	/* add neighbours of v2 into M 
	** if they are also a neighbour of v1*/
	for(i=0; i<size; i++)
	{
		if(A[v2][i]==A[v1][i])
		{
			M[i]=A[v2][i];
		}
	}

	for(i=0; i<size; i++)
	{
		if(A[v2][i]==1)
		{
			if(visited[i]==0)
			{
				traverse(W, visited, i, A, M, v1, size);
			}
		}
	}
	/** TEST::
	for(i=0; i<size; i++)
		printf("W[%d] is %d\n", i, W[i]);
*/	

	/** TEST:: 
	for(i=0; i<size; i++)
		printf("M[%d] is %d\n", i, M[i]);
*/	
	
	/* step 0 */
	x=0;
	y=0;
	/* if vertex i has no neighbour j */
	for(i=0; i<size; i++)
	{
		if(M[i]==1)
		{
			for(j=i+1; j<size; j++)
			{
				/* non-adjacent and both in M */
				if((A[i][j]==0) && (M[j]==1))
				{
					x=i;
					y=j;
					i=size;
					break;
				}
			}
		}
	}
	/***TEST::
	printf("%d and %d\n", x,y);
	*/
	if(y==x) /* both 0 */
	{
		k=0;
		for(i=0;i<size; i++)
		{
			if(M[i]==1)
			{
				Cl[k]=i;
				k++;
			}
		/* M is a clique separating v1 from v2 */
		}
		printf("FIRST ONE\n");
		return k;
	}
	/* find a path from vertex v1 to v2 (chordless) and 
	** internal vertices belonging to W */
	P[x]=1;
	found =0;
	for(i=0; i<size; i++)
	{
		/* only via vertices in M */
		if((W[i]==1) && (A[x][i]==1))
		{
			P[i]=1;
			if(extendPath(P, i, y, A, W, size)==1)
			{
				found =1;
				break;
			}
			P[i]=0;
		}
	}
	if(found ==0)
	{
		fprintf(stderr, "Path finding failed\n");
		exit(1);
	}
	/* add vertex v1 to make a cycle */
	P[v1]=1;

	/* TEST:: 
	for(i=0; i<size; i++)
		printf("%d is vertex %d\n", P[i], i);
	*/

	/*STEP 1 */
	j=0;
	for(i=0; i<size; i++)
	{
		S[i]=P[i];
		j+=P[i];	
	}
	//printf("%d %d\n", j, LEN);
	if(j<LEN)
	{
		k=0;
		for(i=0; i<size; i++)
			if(P[i]==1)
			{
				Cl[k]=i;
				k++;
			}
			//	printf("Clique %d\n", i);
		printf("SECOND ONE\n");
		return k; /* cutset clique of less than four vertices */
	}
	
	SSize =j;
	while(SSize < size)
	{
	/*1. C is component of G(V-S) and R is set of vertices in S with at least one neighbour in C*/
		
		/* don't use neighbours of S for forming R*/
		for(i=0; i<size; i++)
		{
			visited[i]=S[i];
		}
		/* re-initialise C and R to empty list */
		for(i=0; i<size; i++)
		{
			R[i]=0;
			C[i]=0;
		}

		/* find C  and R*/
		for(i=0; i<size; i++)
		{
			/* start our component here */
			if(visited[i]==0)
			{
				/* get component C and at same time let
				** R = vertices in S with at least one neighbour
				** in C */
				traverse2(C, visited, i, A, S, R, size);
				/* we will only get the vertices in the component containing vertex i */
				break;
			}
		}
		/**TEST::
		for(i=0; i<size;i++)
		{
			if(C[i]==1)
				printf("%d\n", i);
		}
		*/

		/**TEST:: 
		for(i=0; i<size;i++)
		{
			if(R[i]==1)
				printf("%d\n", i);
		}*/

		/* STEP 2 */
		x=0;
		y=0;
		/* find non-adjacent vertices x and y in R */
		for(i=0; i<size;i++)
		{
			/* x is in R */
			if(R[i]==1)
			{
				for(j=i+1; j<size; j++)
				{
					if((R[j]==1) && (A[i][j]==0))
					{
						x=i;
						y=j;
						i=size;
						break;
					}
				}
			}
		}
		/* R is a clique*/
		if((x==0) & (y==0))
		{
			k=0;
			for(i=0; i<size; i++)
				if(R[i]==1)
				{
					Cl[k]=i;
					k++;
				}
			printf("THIRD ONE\n");
			return k;
		}
		/** TEST::
		printf("%d and %d\n", x,y);
		*/

		/* else find a chordless x,y path P that contains
		** only vertices in C as intermediate vertices */
		/* init P */
		for(i=0; i<size; i++)
		{
			P[i]=0;
		}
		P[x]=1;
		found =0;
		for(i=0; i<size; i++)
		{
			/* only via vertices in C */
			if((C[i]==1) && (A[x][i]==1))
			{
				P[i]=1;
				if(extendPath(P, i, y, A, C, size)==1)
				{
					found =1;
					break;
				}
				P[i]=0;
			}
		}
		if(found ==0)
		{
			fprintf(stderr, "Path finding failed\n");
			exit(1);
		}

		/* TEST::
		for(i=0; i<size; i++)
		{
			if(P[i]==1)
				printf("%d\n", i);
		}
		*/
		/* step 3 - adjoin intermediate vertices on P to S */
		for(i=0; i<size; i++)
		{
			if(P[i]==1)
			{
				if((i!=x) && (i!=y))
				{
					S[i]=1;
					SSize++;
				}
			}
		}
		/* TEST::
		for(i=0; i<size; i++)
		{
			if(S[i]==1)
				printf("%d\n", i);
		}
		*/
		
		
	}
	return 0;/* no clique cutsets */
}

/* 
Function:	extendPath
Description:	Function attempts to extend path and returns true if possible 
Modified:	Zoe Bukovac	12th May 2011
		Removed fixed size of graphs
*/
int extendPath(int P[], int start, int finish, int **A, int M[], int size)
{
	int i;
	int j;
	int flag =0;

	/* found */
	if(A[start][finish]==1)
	{
		P[finish]=1;
		return 1;
	}

	for(i=0; i<size; i++)
	{
		flag =0;
		/* adjacent to start and in M */
		if((M[i]==1) && (A[start][i]==1))
		{
			/* is it adjacent to anything else in the path */
			for(j=0; j<size; j++)
			{
				if((j!=start) && (P[j]==1))
				{
					if(A[i][j]==1)
					{
						flag =1;
						break;
					}
				}
			}
			/* no chords */
			if(flag ==0)
			{
				P[i]=1;
				if(extendPath(P, i, finish, A, M, size)==1)
				{
					return 1;
				}
				else
				{
					/* try another path */
					P[i]=0;
				}
			}
		}
	}
	return 0;

}

/* 
Function:	Traverse2
Description:	Function computes a DFS traversal as per Traverse. But saves additional information with regards  S and R
Modified:	Zoe Bukovac	12th May 2011
		Removed fixed size of graphs
*/
void traverse2(int C [], int visited[], int v, int **A, int S[], int R[], int size)
{
	int i;

	if(visited[v]==0)
	{
		/* put in C */
		C[v]=1;
		visited[v]=1;
		/* for each neighbour of v, if it is in S, add it to R */
		for(i=0; i<size; i++)
		{
			if(A[v][i]==1)
			{
				if(S[i]==1)
				{
					R[i]=1;
				}
				if(visited[i]==0)
				{
					traverse2(C, visited, i, A, S, R, size);
				}
			}
		}
	}
}


/* 
Function:	Traverse
Description:	Function computes a DFS traversal beginning from cur vertex in graph 
Modified:	Zoe Bukovac	12th May 2011
		Removed fixed size of graphs
*/
void traverse(int W[], int visited[], int cur, int **A, int M[], int v1, int size)
{
	int i;
	
	if(visited[cur]==0)
	{
		/* put in W */
		W[cur]=1;
		/* if cur is a neighbour of v1 too */
		//if(A[cur][v1]==1)
		//{
		//	M[cur]=1;	
		//}
		visited[cur]=1;
		for(i=0; i<size; i++)
		{
			if(A[cur][i]==1)
			{
				if(visited[i]==0)
				{
					traverse(W, visited, i, A, M, v1, size);
				}
				/* cur is adjacent to i and in W 
				** and i is adjacent to v1 add it */
				else if(A[v1][i]==1)
				{
					M[i]=1;
				}

			}
		}
	}

}

