/*

File:	 	graph.c
Description: 	Handles all of the graph creation, copying and other graph operations

TODO:				
*/
#include <stdlib.h>
#include <stdbool.h>
#include "iso.h"
#include "graph.h"

void testIsClique(void);
int innerProduct(int* array1, int* array2, int size);

//Structure for storing an adjacency matrix graph
struct graph
{
	int numVertices;	//Number of vertices
	int numEdges;		//Number of edges
	int **adjMtrx;		//The adjacentcy matrix of the graph
	int *degree;		//The degrees of each vertex in the graph
	int id;
	int cannonID;
	int clique;
	int references;
	char desc[20];		//A description of the edges removed and added to get this graph
				//relative to the original graph in the certificate.
};

void freeGraph(graph inpGraph)
{
	//printf("\nFREEING GRAPH G%d\n",inpGraph->id);
	int i = 0;
	
	for(i = 0; i < inpGraph->numVertices; i++)
	{
		free(inpGraph->adjMtrx[i]);
	}
	free(inpGraph->adjMtrx);
	free(inpGraph->degree);
	free(inpGraph);
}

void setID(graph graph, int id)
{
	graph->id = id;
}
int getID(graph inpGraph)
{
	return inpGraph->id;
}

void incReferences(graph inpGraph)
{
	inpGraph->references++;
	//printf("\nINC G%d REFS: NOW %d\n",inpGraph->id,inpGraph->references);
}

void decReferences(graph inpGraph)
{
	inpGraph->references--;
	//printf("\nDEC G%d REFS: NOW %d\n",inpGraph->id,inpGraph->references);
}

int getReferences(graph inpGraph)
{
	return inpGraph->references;
}

graph createGraphRef(graph inpGraph)
{
	incReferences(inpGraph);
	return inpGraph;
}

void destroyGraphRef(graph *inpRef)
{
	//Grab the graph that this reference points to
	graph theGraph = *inpRef;
	//Decrement the number of references to the graph
	decReferences(theGraph);
	//If the are no remaining references
	if(theGraph->references == 0)
	{
		//printf("\nFreeing graph %d",theGraph->id);
		//Free the graph struct
		freeGraph(theGraph);
		//Then set the inital reference to NULL
		*inpRef = NULL;
	}
}

int getRefCount(graph inpGraph)
{
	return inpGraph->references;
}

/*Print the graph's adjacency matrix*/
void printGraphDiagnostics(graph inpGraph)
{
	int i;						//loop index
	int j;						//loop index
	int graphSize;					//number of vertices in the graph
	graphSize = inpGraph->numVertices;
	printf("Graph size is %d\n",graphSize);
	
	
	for (i=0; i<graphSize; i++)
	{
		for (j=0; j<graphSize; j++)
		{
			printf("&Graph[%d][%d] = %p\n", i, j, &inpGraph->adjMtrx[i][j]);
		}
		printf ("\n");
	}
	for (i=0; i<graphSize; i++)
	{
		for (j=0; j<graphSize; j++)
		{
			printf("Graph[%d][%d] = %2d\n", i, j, inpGraph->adjMtrx[i][j]);
		}
		printf ("\n");
	}
}

/*Prints a list of the graph's vertices, each with a list of adjacent vertices*/
void printGraph(graph inpGraph)
{
	int i;							//loop index
	int j;							//loop index
	int graphSize;					//number of Vertices in newGraph
	graphSize = inpGraph->numVertices;
	printf("\nG%d%s: Graph order: %d\n",inpGraph->id,inpGraph->desc,graphSize);
	
	
	
	//print newGraph
	for (i=0; i<graphSize; i++)
	{
		printf("Vertex: %d Degree: %d Edges: ", i, inpGraph->degree[i]);
		for (j=0; j<graphSize; j++)
		{
			if(i!=j && inpGraph->adjMtrx[i][j]==1)
			printf(" %d",j);
		}
		printf ("\n");
	}
}

void printGraphToFile(graph inpGraph, FILE *outFile)
{
	int i;							//loop index
	int j;							//loop index
	int graphSize;					//number of Vertices in newGraph
	graphSize = inpGraph->numVertices;
	fprintf(outFile,"\nG%d%s: Graph order: %d\n",inpGraph->id,inpGraph->desc,graphSize);
	
	
	
	//print newGraph
	for (i=0; i<graphSize; i++)
	{
		fprintf(outFile,"Vertex: %d Degree: %d Edges: ", i, inpGraph->degree[i]);
		for (j=0; j<graphSize; j++)
		{
			if(i!=j && inpGraph->adjMtrx[i][j]==1)
			fprintf(outFile," %d",j);
		}
		fprintf(outFile,"\n");
	}
}
/*TODO
Graph createGraph(FILE file)
{
}
*/

int isCliqueSeparable(graph inpGraph, int *clique)
{
	return isCliqueSep(inpGraph->adjMtrx, inpGraph->numVertices, clique);
}

//TODO This will eventually need to take a pre-validated file to build the graph from
//At the moment it just a sparseish graph on 10 vertices
graph createGraph(int graphSize)
{
	//Read graph from file
		//get filename and open
		//setup appropriate size matrix
		//mark edges in matrix

	int i; 					//loop index
	int j; 					//loop index
	graph createdGraph = NULL;		//the new graph
	int n;					//the number of vertices in the graph

	n = graphSize;
	//TODO refs need an intit.	

	createdGraph = malloc(sizeof(* createdGraph));
	
	//TODO add in checks for null pointers
	//allocate room for an array of pointers
	createdGraph->adjMtrx = malloc(n * sizeof(int*)); 

	//and for each of those, allocate room for an array of ints
	for (i=0; i<n; i++)
	{
		createdGraph->adjMtrx[i] = malloc(n * sizeof(int));
	}

	
	for (i=0; i<n; i++)
	{
		for (j=0; j<n; j++)
		{	
		
			createdGraph->adjMtrx[i][j] = 0;
			/*build a k_n
			if(i!=j)
			{
				graph->adjMtrx[i][j] = 1;
			}
			else
			{
				createdGraph->adjMtrx[i][j] = 9;
			}*/
		}
	}
	
	createdGraph->degree = malloc(n * sizeof(int)); 	
	
	addEdge(createdGraph,0,1);
	addEdge(createdGraph,1,2);
	addEdge(createdGraph,2,0);
	addEdge(createdGraph,3,0);
	addEdge(createdGraph,4,1);
	addEdge(createdGraph,4,5);
	addEdge(createdGraph,2,5);
	//addEdge(createdGraph,5,3);

	//addEdge(createdGraph,8,6);
	//addEdge(createdGraph,7,6);
	//addEdge(createdGraph,6,3);
	//addEdge(createdGraph,6,0);
	//addEdge(createdGraph,8,9);
	//addEdge(createdGraph,8,3);
	//addEdge(createdGraph,4,3);
	//addEdge(createdGraph,6,4);
	//addEdge(createdGraph,4,0);
	//addEdge(createdGraph,4,1);
	//addEdge(createdGraph,6,1);
	
	//TODO this should probably go...
	/*
	//print matrix contents
	for (i=0; i<n; i++)
	{
		for (j=0; j<n; j++)
		{
			printf("createdGraph[%d][%d] = %2d\n", i, j, createdGraph.adjMtrx[i][j]);
		}
		printf ("\n");
	}
	
	//TODO this should probably go...
	/* now for each pointer, free its array of ints */
	/*for (i = 0; i < size_y; i++)
	{
		free(createdGraph.adjMtrx[i]);
	}
	/* now free the array of pointers
	free(createdGraph);*/
	
	
	//Set the number of vertices and edges
	createdGraph->numVertices = n;
	createdGraph->numEdges = 10; //TODO will need to know this eventually
	createdGraph->id = 0;
	createdGraph->references = 1;
	
	return createdGraph;
}

graph initGraph(int size)
{
	
	int i,j;
	graph createdGraph = malloc(sizeof(* createdGraph));
	createdGraph->degree = malloc(size * sizeof(int));
	
	//Allocate space for an array of *ints
	createdGraph->adjMtrx = malloc(size * sizeof(int*));
	//And for each allocate room for an array of ints
	for (i=0; i<size; i++)
	{
		createdGraph->adjMtrx[i] = malloc(size * sizeof(int));
		
		//Initialise the degree counters while we're here
		createdGraph->degree[i] = 0;	
	}

	for (i = 0; i < size; i++)
	{
		for (j = 0; j < size; j++)
		{
			createdGraph->adjMtrx[i][j] = 0;	
		}	
	}
	
	createdGraph->numVertices = size;
	createdGraph->numEdges = 0;
	createdGraph->id = 0;
	createdGraph->cannonID = 0;
	createdGraph->references = 1;
	createdGraph->desc[0] = '\0';
	return createdGraph;
}

void setCannonID(graph inpGraph, int id)
{
	inpGraph->cannonID = id;
}

int getCannonID(graph inpGraph)
{
	return inpGraph->cannonID;
}


//Make a copy of the given oldGraph at newGraph 
graph copyGraph(graph oldGraph)
{
	int i, j;
	int size = oldGraph->numVertices;
	graph newGraph = NULL;
	
	newGraph = malloc(sizeof(*newGraph));
	if(newGraph==NULL) printf("\nFUCKED THE YAK\n");
	newGraph->numVertices = oldGraph->numVertices;
	newGraph->numEdges = oldGraph->numEdges;
	newGraph->degree = malloc(size * sizeof(int));
	
	//Allocate space for the newGraph's adjMtrx
	newGraph->adjMtrx = malloc(size * sizeof(int *));
	
	for (i = 0; i < size; i++)
	{
		newGraph->adjMtrx[i] = malloc(size * sizeof(int));
		if(newGraph->adjMtrx[i]==NULL) printf("\nFUCKED THE YAK\n");
	}
	
	//Copy the adjMtrx
	for(i = 0; i < size; i++)
	{
		for(j = 0; j < size; j++)
		{
			newGraph->adjMtrx[i][j] = oldGraph->adjMtrx[i][j];
		}
		newGraph->degree[i] = oldGraph->degree[i];
	}
	
	newGraph->references = 1;
	return newGraph;
}

graph copyAndDeleteEdge(graph oldGraph, int u, int v)
{
	graph newGraph = copyGraph(oldGraph);
	newGraph->adjMtrx[u][v] = 0;
	newGraph->adjMtrx[v][u] = 0;
	newGraph->degree[v]--;
	newGraph->degree[u]--;
	newGraph->numEdges--;
	sprintf(newGraph->desc,"{G%d-(%d,%d)}",oldGraph->id,u,v);

	return newGraph;
	
}

graph copyAndAddEdge(graph oldGraph, int u, int v)
{
	graph newGraph = copyGraph(oldGraph);
	newGraph->adjMtrx[u][v] = 1;
	newGraph->adjMtrx[v][u] = 1;
	newGraph->degree[v]++;
	newGraph->degree[u]++;
	newGraph->numEdges++;
	sprintf(newGraph->desc,"{G%d+(%d,%d)}",oldGraph->id,u,v);
	return newGraph;
}

//Delete an edge from the graph
void deleteEdge(graph inpGraph, int u, int v)
{
	inpGraph->adjMtrx[u][v] = 0;
	inpGraph->adjMtrx[v][u] = 0;
	inpGraph->degree[v]--;
	inpGraph->degree[u]--;
	inpGraph->numEdges--;
}

//Add and edge to the graph
void addEdge(graph graph, int u, int v)
{	
	if(!isEdge(graph, u, v))	
	{
		graph->degree[v]++;
		graph->degree[u]++;
		graph->numEdges++;
	}
	graph->adjMtrx[u][v] = 1;
	graph->adjMtrx[v][u] = 1;
}

//NOT the same as delete edge. Used during contraction ops
void addNonEdge(graph graph, int u, int v)
{
	graph->adjMtrx[u][v] = 0;
	graph->adjMtrx[v][u] = 0;
}


//Identify two Vertices in the graph
/*void identifyEdge(graph oldGraph, graph newGraph, int u, int v)
{
	contractEdge(oldGraph,u,v);
}*/

//Contract an edge in the graph
//TODO SOMETHING IS WRONG WITHE THE CONTRACTION CODE. FIX IT!!!!!!
//Remember that the newGraph is always smaller than the oldGraph.

graph copyAndContractEdge(graph oldGraph, int u, int v)
{
	int i,j;
	int temp;
	graph newGraph = initGraph(oldGraph->numVertices-1);
	
	for (i = 0; i < newGraph->numVertices; i++)
	{
		for (j = 0; j < newGraph->numVertices; j++)
		{
			newGraph->adjMtrx[i][j] = 0;	
		}	
	}
	
	sprintf(newGraph->desc,"{G%d/(%d,%d)}",oldGraph->id,u,v);
	//make sure u has an earlier entry in the matrix than v (i.e. ensure u < v)
	if(u>v)
	{
		temp = u;
		u = v;
		v = temp;
	}
	
	//Initialise the newGraph's degree array
	for(i = 0; i < oldGraph->numVertices-1; i++)
	{
		newGraph->degree[i] = 0;
	}
	
	for(i = 0; i < oldGraph->numVertices; i++)
	{	
		for(j = 0; j < i; j++)
		{
			
			if(i == v)
			{
				//Make sure that the entries in v are added into u, and there are no loops
				if(oldGraph->adjMtrx[v][j]==1 && j != u)// && oldGraph->adjMtrx[i][u]==0)
				{
					//if there was an edge here for v, 
						//add one to u.
					addEdge(newGraph,u,j);
				}
			}
			else if(i > v)
			{
				//if we've gone past the v row in the oldGraph...
				if(j==v && oldGraph->adjMtrx[i][j]==1)
				{
						//and we're at some other vertex > v
						//and it had an edge to v, 
							//we need to add an edge to u instead.
						addEdge(newGraph,(i-1),u);
				}
				else if(j > v)
				{
					//if we're in the lower right quadrant, move all vaues up by one diagonally
					if(oldGraph->adjMtrx[i][j]==1)
					{
						addEdge(newGraph,(i-1),(j-1));
					}
					else
					{
						addNonEdge(newGraph,(i-1),(j-1));
					}
				}
				else
				{
					//otherwise, copy values shifted up by one row
					if(oldGraph->adjMtrx[i][j]==1)
					{
						addEdge(newGraph,(i-1),j);
					}
					else
					{
						addNonEdge(newGraph,(i-1),j);
					}
				}
			}
			else
			{
				//otherwise, this location is unchanged from the oldGraph, so copy directly
				if(oldGraph->adjMtrx[i][j]==1)
				{
					addEdge(newGraph,i,j);
				}
				else
				{
					addNonEdge(newGraph,i,j);
				}
			}
		}
	}
	//TODO modifiy the description
	return newGraph;
}


graph copyAndContractEdge1(graph oldGraph, int u, int v)
{
	int i = 0, j = 0, max = oldGraph->numVertices-1;
	int temp;
	graph newGraph = initGraph(oldGraph->numVertices-1);
	
	//Make sure u is the earlier entry in the matrix than v (i.e. ensure u < v)
	if(u>v)
	{
		temp = u;
		u = v;
		v = temp;
	}
	
	//Initialise the newGraph's degree array
	for(i = 0; i < oldGraph->numVertices-1; i++)
	{
		newGraph->degree[i] = 0;
	}
	
	for(i = 0; i < oldGraph->numVertices; i++)
	{	
		for(j = 0; j < i; j++)
		{
		
			if(i == v)
			{
				if(oldGraph->adjMtrx[i][j]==1) //&& oldGraph->adjMtrx[i][u]==0)
				{
					//if there no edge here in u,
					//and there was one for v, 
						//add one for the current vertex
					addEdge(newGraph,u,j);
				}
				else
				{
					addNonEdge(newGraph,u,j);
				}
			}
			else if(i == max)
			{
				//Were at the last entry in the oldGraph adjMtrx, so copy to where v was
				if(oldGraph->adjMtrx[i][j]==1)
				{
					addEdge(newGraph,v,j);
				}
				else
				{
					addNonEdge(newGraph,v,j);
				}
			}
			else
			{
				//otherwise, this location is unchanged from the oldGraph, so copy directly
				if(oldGraph->adjMtrx[i][j]==1)
				{
					addEdge(newGraph,i,j);
				}
				else
				{
					addNonEdge(newGraph,i,j);
				}
			}
		}
	}
	//TODO modifiy the description
	sprintf(newGraph->desc,"{G%d/(%d,%d)}",oldGraph->id,u,v);
	return newGraph;
}
//Finds G1 and G2 where G=G1+G2/Kr
void unglue(graph *g, graph *g1, graph *g2, int clique[], int cliqueSize)
{
	
}


bool isValidVertex(graph graph, int i)
{
	return i >= 0 && i < graph->numVertices;
}

int isEdge(graph graph, int u, int v)
{
	if(isValidVertex(graph, u) && isValidVertex(graph, v) && graph->adjMtrx[u][v]==1)
	{
		return 1;
	}
	return 0;
}



//CAN REMOVE WHEN KERRI'S CODE IS FIXED
/*Removes the degree counters from the diagonal indicies in the adjMatrix so Kerri's code will be able to use the graph*/
/*void convertToKerriGraph(graph graph)
{
	int i;
	for(i = 0; i < graph->numVertices; i++)
	{
		graph->adjMtrx[i][i]=0;
	}
}

//CAN REMOVE WHEN KERRI'S CODE IS FIXED ** NOT BEING USED AT THIS POINT
/*Replaces the degree counters from the diagonal indicies in the adjMatrix*/
/*void revertToZoeGraph(graph graph)
{
	int i,j,currentCount = 0;
	for(i=0; i<graph->numVertices; i++)
	{
		for(j=0; j<graph->numVertices; j++)
		{
			if(graph->adjMtrx[i][j]==1)
			{
				currentCount++;
			}
		}
		graph->adjMtrx[i][i] = currentCount;
		currentCount = 0;
	}
}*/

int getSize(graph graph)
{
	return graph->numVertices;
}
int getNumEdges(graph graph)
{
	return graph->numEdges;
}
char * getDesc(graph graph)
{
	return graph->desc;
}
/*
void separateOnClique(Graph graph, Graph h1, Graph h2, int *clique)
{
	//Make a copy of the graphs adjMtrx and remove the clique
	int size = graph->numVertices;
	int **temp = malloc(size * sizeof(int *));
	int i,j;

	for (i=0; i<size; i++)
	{
		temp[i] = malloc(size * sizeof(int));
	}
	
	//Traverse the first component of the graph and store it in h1
	for(i=0; i<size; i++)
	{
		for(j=0; j<size&&j!=-1; j++)
		{
			//TEST
			printf("i:%d j:%d",i,j);
			//If at the first vertex not in the clique...
			if(i!=j)
			{
				//Grab this component
				getComponent(graph,h1,clique,int,i);
			}
		}
	}
	
	//All other verticies must belong to h2
}

void getComponent(Graph graph, Graph h1, int *clique, int start)
{
	int size = graph->numVertices;
	int visited[size];
	int **temp = malloc(size * sizeof(int *));
	int i,j;

	for (i=0; i<size; i++)
	{
		temp[i] = malloc(size * sizeof(int));
	}

	temp[][]
	for(int i=0; i<size; i++)
	{
		
	}
}
*/



/******************************************************************************
Function:	equalGraphs
Description:	Checks if the two graphs have the same adjacency matrix
Inputs:		g1, g2 - the graphs
Return Value:	1 if they are the same, 0 if they are not
Side Effects:	None
Created:	Z.Bukovac Feb 2012
******************************************************************************/
int equalGraphs(graph g1, graph g2)
{
	int i, j;
	if(g1->numVertices != g2->numVertices)
	{
		return 0;
	}

	for(i = 0; i < g1->numVertices; i++)
	{
		for(j = 0; j < g1->numVertices; j++)
		{
			if(g1->adjMtrx[i][j] != g2->adjMtrx[i][j])
			{
				return 0;
			}
		}
	}
	return 1;
}
/******************************************************************************
Function:	isClique
Description:	Checks if the given subset of vertices are a clique in the graph
Inputs:		graph - the graph
		vertexSet - the subset of vertices to be checked
		setSize - the number of vertices in the vertexSet
Return Value:	1 if vertexSet is a clique, 0 if it is not, -1 if there is an error
Side Effects:	None
Created:	Z.Bukovac Jan 2012
******************************************************************************/
int isClique(graph graph, int* vertexSet, int setSize)
{
	int i,j;	//Array indices

	if(setSize > graph->numVertices)
	{
		//Error: vertexSet is larger than the graph
		return -1;
	}

	//For each vertex in the set...
	for(i = 0; i < setSize; i++)
	{
		if(vertexSet[i] > graph->numVertices)
		{
			//Error: invalid indices
			return -1;
		}

		//Check that it is adjacent to the remainder of the set
		for(j = i+1; j < setSize; j++)
		{
			if(vertexSet[j] > graph->numVertices)
			{
				//Error: invalid indices
				return -1;
			}

			if(graph->adjMtrx[vertexSet[i]][vertexSet[j]]!=1)
			{
				//If not adjacent, this cannot be a clique
				return 0;
			}
		}
	}
	return 1;
}


/******************************************************************************
*** Function:    isClique
*** Description: Checks if the given subset of vertices are a clique in the graph
*** Inputs:      graph - the graph
***              vertexSet - the subset of vertices to be checked
***              setSize - the number of vertices in the vertexSet
*** Return Value: 1 if vertexSet is a clique, 0 if not.
*** Side Effects: None
*** Created:	  Z.Bukovac Jan 2012
******************************************************************************/
/*int isClique(Graph graph, int* vertexSet, int setSize)
{
	int i,j;

	//For each vertex in the set...
	for(i = 0; i < setSize; i++)
	{
		//Check that it is adjacent to the remainder of the set
		for(j = i+1; j < setSize; j++)
		{
			if(graph->adjMtrx[vertexSet[i]][vertexSet[j]]!=1)
			{
				//If not adjacent, this cannot be a clique
				return 0;
			}
		}
	}
	return 1;
}*/


/******************************************************************************
Function:	innerProduct
Description:	Finds the inner product of the two arrays
Inputs:		array1, array2 - the arrays to find the inner product of
		size - the size of the arrays
Return Value:	The inner product of the two arrays
Side Effects:	None
Created:	Z.Bukovac Jan 2012
******************************************************************************/
int innerProduct(int* array1, int* array2, int size)
{
	int i;		//Array index
	int sum = 0;	//Summed return value
	for(i = 0; i<size; i++)
	{
		sum += array1[i] * array2[i];

	}
	return sum;
}

/******************************************************************************
Function:	isomorphic
Description:	Determines if the two given graphs are isomorphic
Inputs:		g1, g2 - the two graphs to check
Return Value:	1 if the graphs are isomorphic, 0 if not
Side Effects:	None
Created:	Z.Bukovac Jan 2012
******************************************************************************/
int isomorphic(graph g1, graph g2)
{
	if(g1->numVertices != g2->numVertices)
	{
		return 0;
	}
	else
	{
		return isIso(g1->adjMtrx, g2->adjMtrx, g2->numVertices);
	}
}



/****************************************************************************/
/*****************************TEST FUNCTIONS*********************************/
/****************************************************************************/

void graphTest(void)
{
	int u = 3;
	int v = 4;
	graph newGraph = initGraph(5);
	printf("\nOriginal Graph:\n===============\n");
	addEdge(newGraph,0,1);
	addEdge(newGraph,1,2);
	addEdge(newGraph,2,3);
	addEdge(newGraph,3,4);
	addEdge(newGraph,4,0);
	printGraph(newGraph);
	
	printf("\nOld Contraction:\n===============\n");
	graph contGraph = copyAndContractEdge(newGraph,u,v);
	printGraph(contGraph);
	
	freeGraph(contGraph);
	printf("\nNew Contraction:\n===============\n");
	contGraph = copyAndContractEdge1(newGraph,u,v);
	printGraph(contGraph);
}
void test(void)
{
	int i;							//loop index
	int j;							//loop index
	int graphSize;					//number of vertices in newgraph
	graph graph, newGraph , g1, g2;	//some Graphs
	
	
	//Create and print a new graph
	graph = createGraph(10);
	graphSize = graph->numVertices;
	printGraph(graph);
	//printGraphDiagnostics(graph);
	newGraph = copyAndContractEdge(graph,6,4);
	printGraph(newGraph);
	
	//cs1(&graph,&g1,&g2,6,4);
	g1 = graph;
	g2 = copyGraph(g1);
	deleteEdge(g2, 8, 6);
	//convertToKerriGraph(g1);
	//convertToKerriGraph(g2);
	//printGraph(g1);
	//printGraph(g2);
	i = isIso(g1->adjMtrx, g2->adjMtrx, g1->numVertices);
	if(i)
	{
		printf("ISOMORPHIC\n");
	}
	else
	{
		printf("NOT ISOMORPHIC\n");
	}
	i = isIso(g1->adjMtrx, g1->adjMtrx, g1->numVertices);
	if(i)
	{
		printf("ISOMORPHIC\n");
	}
	else
	{
		printf("NOT ISOMORPHIC\n");
	}
	i = isIso(g2->adjMtrx, g2->adjMtrx, g2->numVertices);
	if(i)
	{
		printf("ISOMORPHIC\n");
	}
	else
	{
		printf("NOT ISOMORPHIC\n");
	}

	testIsClique();
}

void testIsClique(void)
{

	graph graph = createGraph(6);
	int *clique = malloc(3 * sizeof(int));
	clique[0]=0;
	clique[1]=1;
	clique[2]=4;
	printf("Clique?0: %d\n",isClique(graph, clique, 3));

	clique[0]=0;
	clique[1]=1;
	clique[2]=2;
	printf("Clique?1: %d\n", isClique(graph, clique, 3));

	clique[0]=1;
	clique[1]=0;
	printf("Clique?1: %d\n", isClique(graph, clique, 3));

	clique[0]=1;
	clique[1]=0;
	clique[2]=2;
	printf("Clique?1: %d\n", isClique(graph, clique, 3));

	clique[0]=1;
	clique[1]=8;
	clique[2]=2;
	printf("Clique?-1: %d\n", isClique(graph, clique, 3));

	free(clique);


	clique =  malloc(8 * sizeof(int));
	clique[0]=1;
	clique[1]=1;
	clique[2]=1;
	clique[3]=1;
	clique[4]=1;
	clique[5]=1;
	clique[6]=1;
	clique[7]=1;
	printf("Clique?-1: %d\n", isClique(graph, clique, 8));


	printf("IP of clique = %d",innerProduct(clique,clique,8));

	free(clique);
}
